/*
Copyright 2023 Alexandre Trofimov

This file is part of getcdmach external ballistics software.

getcdmach is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

getcdmach is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with getcdmach.  If not, see <http://www.gnu.org/licenses/>.
*/

function changeDmm() {
    document.getElementById("bulletDin").value = UC.to(UC.from(document.getElementById("bulletDmm").value, "mm"), "inch");
}

function changeDin() {
    document.getElementById("bulletDmm").value = UC.to(UC.from(document.getElementById("bulletDin").value, "inch"), "mm");
}

function changeMgm() {
    document.getElementById("bulletMgr").value = UC.to(UC.from(document.getElementById("bulletMgm").value, "gram"), "grain");
}

function changeMgr() {
    document.getElementById("bulletMgm").value = UC.to(UC.from(document.getElementById("bulletMgr").value, "grain"), "gram");
}

function goDoWork() {
    var cal = document.getElementById("bulletDmm").value;
    var mass = document.getElementById("bulletMgm").value;

    var veltab = document.getElementById("dvtext").value;
    var unitD, unitV;
    if (document.getElementById("inputUnitsModern").checked) {
        unitD = "m";
        unitV = "m/s";
    }
    else {
        unitD = "yard";
        unitV = "fps";
    }

    // parse dist/vel table
    var vtcount = 0;
    var vtdata = [];

    var d;
    var v;
    var dv;
    var lspattern = /^(\d*[\.,]?\d*)[\t\s;]+(\d*[\.,]?\d*)$/;
    veltab = veltab.replaceAll(",", ".");
    var lines = veltab.split('\n');
    for (let i = 0; i < lines.length; i++) {
        dv = lines[i].match(lspattern);
        if (dv) {
            d = parseFloat(dv[1]);
            v = parseFloat(dv[2]);
            if (isNaN(d) || isNaN(v)) {
                console.log("*** ACHTUNG: failure parsing line " + lines[i]);
                continue;
            }
            vtdata[vtcount] = new VelTabPt(UC.from(d,unitD),UC.from(v,unitV));
            vtcount = vtcount + 1;
        }
    }
    // sort by distance, just in case
    vtdata.sort(function (a,b) {return a.d - b.d});

    if (vtcount < 3) {
        console.log("*** ACHTUNG: not enough valid data points in the velocity table, bombing out.")
        return false;
    }

    var cdmtab = new CdMachTable(vtdata, cal, mass);

    var outtext = "";
    if (document.getElementById("outCalc").checked) {
        outtext = "Calculated<br /><br />";
        for (let i = 0; i < cdmtab.calculated.length; i = i + 1) {
            outtext = outtext + cdmtab.calculated[i].mach + ";" + cdmtab.calculated[i].cd + "<br />";
        }
        outtext = outtext + "<br /><br />";
    }

    outtext = outtext + "Interpolated<br /><br />";
    // outtext = "";
    for (let i = 0; i < cdmtab.interpolated.length; i = i + 1) {
        outtext = outtext + cdmtab.interpolated[i].mach + ";" + cdmtab.interpolated[i].cd + "<br />";
    }
    document.getElementById("outText").innerHTML = outtext;


} // function goDoWork()

document.getElementById("bulletDmm").addEventListener("change", changeDmm);
document.getElementById("bulletDin").addEventListener("change", changeDin);
document.getElementById("bulletMgm").addEventListener("change", changeMgm);
document.getElementById("bulletMgr").addEventListener("change", changeMgr);

document.getElementById("bulletDmm").value = 8.61;
changeDmm();
document.getElementById("bulletMgm").value = 16.2;
changeMgm();

document.getElementById("btn-dowork").addEventListener("click", goDoWork);